@@ -3,7 +3,7 @@ package Net::DNS::Packet;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: Packet.pm 1812 2020-10-07 18:09:53Z willem $)[2];
+our $VERSION = (qw$Id: Packet.pm 1886 2022-12-20 11:27:25Z willem $)[2];
 
 
 =head1 NAME
@@ -115,12 +115,12 @@ sub decode {
 	my $self;
 	eval {
 		local $SIG{__DIE__};
-		die 'corrupt wire-format data' if length($$data) < HEADER_LENGTH;
+		my $length = length $$data;
+		die 'corrupt wire-format data' if $length < HEADER_LENGTH;
 
 		# header section
 		my ( $id, $status, @count ) = unpack 'n6', $$data;
 		my ( $qd, $an, $ns, $ar ) = @count;
-		my $length = length $$data;
 
 		$self = bless {
 			id	   => $id,
@@ -138,23 +138,23 @@ sub decode {
 		my $record;
 		$offset = HEADER_LENGTH;
 		while ( $qd-- ) {
-			( $record, $offset ) = decode Net::DNS::Question( $data, $offset, $hash );
+			( $record, $offset ) = Net::DNS::Question->decode( $data, $offset, $hash );
 			CORE::push( @{$self->{question}}, $record );
 		}
 
 		# RR sections
 		while ( $an-- ) {
-			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
+			( $record, $offset ) = Net::DNS::RR->decode( $data, $offset, $hash );
 			CORE::push( @{$self->{answer}}, $record );
 		}
 
 		while ( $ns-- ) {
-			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
+			( $record, $offset ) = Net::DNS::RR->decode( $data, $offset, $hash );
 			CORE::push( @{$self->{authority}}, $record );
 		}
 
 		while ( $ar-- ) {
-			( $record, $offset ) = decode Net::DNS::RR( $data, $offset, $hash );
+			( $record, $offset ) = Net::DNS::RR->decode( $data, $offset, $hash );
 			CORE::push( @{$self->{additional}}, $record );
 		}
 
@@ -162,10 +162,11 @@ sub decode {
 		return unless $self->header->opcode eq 'DSO';
 
 		$self->{dso} = [];
-		while ( $offset < $length ) {
+		my $limit = $length - 4;
+		while ( $offset < $limit ) {
 			my ( $t, $l, $v ) = unpack "\@$offset n2a*", $$data;
 			CORE::push( @{$self->{dso}}, [$t, substr( $v, 0, $l )] );
-			$offset += 4 + $l;
+			$offset += ( $l + 4 );
 		}
 	};
 
@@ -397,12 +398,13 @@ sub string {
 	my $self = shift;
 
 	my $header = $self->header;
+	my $opcode = $header->opcode;
 	my $server = $self->{replyfrom};
 	my $length = $self->{replysize};
 	my $origin = $server ? ";; Response received from $server ($length octets)\n" : "";
 	my @record = ( "$origin;; HEADER SECTION", $header->string );
 
-	if ( $self->{dso} ) {
+	if ( $opcode eq 'DSO' ) {
 		CORE::push( @record, ";; DSO SECTION" );
 		foreach ( @{$self->{dso}} ) {
 			my ( $t, $v ) = @$_;
@@ -411,7 +413,7 @@ sub string {
 		return join "\n", @record, "\n";
 	}
 
-	my @section  = $header->opcode eq 'UPDATE' ? qw(ZONE PREREQUISITE UPDATE) : qw(QUESTION ANSWER AUTHORITY);
+	my @section  = $opcode eq 'UPDATE' ? qw(ZONE PREREQUISITE UPDATE) : qw(QUESTION ANSWER AUTHORITY);
 	my @question = $self->question;
 	my $qdcount  = scalar @question;
 	my $qds	     = $qdcount != 1 ? 's' : '';
@@ -563,7 +565,7 @@ sub _section {				## returns array reference for section
     $query = Net::DNS::Packet->new( 'www.example.com', 'A' );
 
     $query->sign_tsig(
-		'Khmac-sha512.example.+165+01018.private',
+		$keyfile,
 		fudge => 60
 		);
 
@@ -597,12 +599,6 @@ specified key.
     $query->sign_tsig( $tsig );
 
 
-The historical simplified syntax is still available, but additional
-options can not be specified.
-
-    $packet->sign_tsig( $key_name, $key );
-
-
 The response to an inbound request is signed by presenting the request
 in place of the key parameter.
 
@@ -865,7 +861,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
