@@ -3,7 +3,7 @@ package Net::DNS::RR;
 use strict;
 use warnings;
 
-our $VERSION = (qw$Id: RR.pm 1847 2021-08-11 10:02:44Z willem $)[2];
+our $VERSION = (qw$Id: RR.pm 1886 2022-12-20 11:27:25Z willem $)[2];
 
 
 =head1 NAME
@@ -103,16 +103,16 @@ sub _new_string {
 	my ( $owner, @token ) = grep { defined && length } split /$PARSE_REGEX/o;
 
 	croak 'unable to parse RR string' unless scalar @token;
-	my $t1 = uc $token[0];
+	my $t1 = $token[0];
 	my $t2 = $token[1];
 
 	my ( $ttl, $class );
 	if ( not defined $t2 ) {				# <owner> <type>
-		@token = ('ANY') if $classbyname{$t1};		# <owner> <class>
+		@token = ('ANY') if $classbyname{uc $t1};	# <owner> <class>
 	} elsif ( $t1 =~ /^\d/ ) {
 		$ttl   = shift @token;				# <owner> <ttl> [<class>] <type>
 		$class = shift @token if $classbyname{uc $t2} || $t2 =~ /^CLASS\d/i;
-	} elsif ( $classbyname{$t1} || $t1 =~ /^CLASS\d/ ) {
+	} elsif ( $classbyname{uc $t1} || $t1 =~ /^CLASS\d/i ) {
 		$class = shift @token;				# <owner> <class> [<ttl>] <type>
 		$ttl   = shift @token if $t2 =~ /^\d/;
 	}
@@ -201,15 +201,15 @@ sub _new_hash {
 
 =head2 decode
 
-    ( $rr, $next ) = decode Net::DNS::RR( \$data, $offset, @opaque );
+    ( $rr, $next ) = Net::DNS::RR->decode( \$data, $offset, @opaque );
 
 Decodes a DNS resource record at the specified location within a
 DNS packet.
 
 The argument list consists of a reference to the buffer containing
 the packet data and offset indicating where resource record begins.
-Remaining arguments, if any, are passed as opaque data to
-subordinate decoders.
+Any remaining arguments are passed as opaque data to subordinate
+decoders and do not form part of the published interface.
 
 Returns a C<Net::DNS::RR> object and the offset of the next record
 in the packet.
@@ -217,9 +217,6 @@ in the packet.
 An exception is raised if the data buffer contains insufficient or
 corrupt data.
 
-Any remaining arguments are passed as opaque data to subordinate
-decoders and do not form part of the published interface.
-
 =cut
 
 use constant RRFIXEDSZ => length pack 'n2 N n', (0) x 4;
@@ -228,7 +225,7 @@ sub decode {
 	my $base = shift;
 	my ( $data, $offset, @opaque ) = @_;
 
-	my ( $owner, $fixed ) = decode Net::DNS::DomainName1035(@_);
+	my ( $owner, $fixed ) = Net::DNS::DomainName1035->decode(@_);
 
 	my $index = $fixed + RRFIXEDSZ;
 	die 'corrupt wire-format data' if length $$data < $index;
@@ -684,11 +681,12 @@ sub _subclass {
 			my $subclass = join '::', __PACKAGE__, $identifier;
 
 			unless ( eval "require $subclass" ) {	## no critic ProhibitStringyEval
-				push @INC, sub {
-					Net::DNS::Parameters::_typespec("$rrtype.RRTYPE");
-				};
-
+				my $perl = Net::DNS::Parameters::_typespec("$rrtype.RRTYPE");
 				$subclass = join '::', __PACKAGE__, "TYPE$rrtype";
+				push @INC, sub {		# see perldoc -f require
+					my @line = split /\n/, $perl;
+					return ( sub { defined( $_ = shift @line ) } );
+				};
 				eval "require $subclass";	## no critic ProhibitStringyEval
 			}
 
@@ -821,7 +819,7 @@ All rights reserved.
 
 Permission to use, copy, modify, and distribute this software and its
 documentation for any purpose and without fee is hereby granted, provided
-that the above copyright notice appear in all copies and that both that
+that the original copyright notices appear in all copies and that both
 copyright notice and this permission notice appear in supporting
 documentation, and that the name of the author not be used in advertising
 or publicity pertaining to distribution of the software without specific
